@@ -24,7 +24,7 @@ from ._compat import itervalues, string_types, xrange
 from .model import DefaultMeta
 from . import utils
 
-__version__ = "2.4.1"
+__version__ = "2.4.3"
 
 # the best timer function for the platform
 if sys.platform == 'win32':
@@ -498,12 +498,7 @@ class BaseQuery(orm.Query):
         if not items and page != 1 and error_out:
             abort(404)
 
-        # No need to count if we're on the first page and there are fewer
-        # items than we expected.
-        if page == 1 and len(items) < per_page:
-            total = len(items)
-        else:
-            total = self.order_by(None).count()
+        total = self.order_by(None).count()
 
         return Pagination(self, page, per_page, total, items)
 
@@ -699,6 +694,10 @@ class SQLAlchemy(object):
 
     .. versionchanged:: 2.4
        The `use_native_unicode` parameter was deprecated.
+
+    .. versionchanged:: 2.4.3
+        ``COMMIT_ON_TEARDOWN`` is deprecated and will be removed in
+        version 3.1. Call ``db.session.commit()`` directly instead.
     """
 
     #: Default query class used by :attr:`Model.query` and other queries.
@@ -848,6 +847,13 @@ class SQLAlchemy(object):
         @app.teardown_appcontext
         def shutdown_session(response_or_exc):
             if app.config['SQLALCHEMY_COMMIT_ON_TEARDOWN']:
+                warnings.warn(
+                    "'COMMIT_ON_TEARDOWN' is deprecated and will be"
+                    " removed in version 3.1. Call"
+                    " 'db.session.commit()'` directly instead.",
+                    DeprecationWarning,
+                )
+
                 if response_or_exc is None:
                     self.session.commit()
 
